#!/usr/bin/env python
#Copyright (C) 2007 - darkry
#Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.og>

import urllib
from xml.dom import minidom
from commands import getoutput
from time import sleep
from os import path
from sys import exit

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "kiba-weather"

#Check every 5 minutes for new mails
CHECK_INTERVALL = 300000

if getoutput("kiba-settings --get-bool weather_enable") != "TRUE":
	exit()

USE_METRIC = getoutput("kiba-settings --get-bool weather_metric_system")
if USE_METRIC == "Unable to find Option 'weather_metric_system' in the Settings File" or USE_METRIC == "TRUE":
	USE_METRIC = 1
else:
	USE_METRIC = 0
ZIP = getoutput("kiba-settings --get-char weather_zip_code")
if ZIP == "Unable to find Option 'weather_zip_code' in the Settings File":
	ZIP = 'GMXX0014'
BROWSER = getoutput("kiba-settings --get-char weather_browser")
if BROWSER == "Unable to find Option 'weather_browser' in the Settings File":
	BROWSER = 'firefox'

URL = "http://www.weather.com/weather/local/"
WEATHER_URL = 'http://xml.weather.yahoo.com/forecastrss?p=' + ZIP
WEATHER_NS = 'http://xml.weather.yahoo.com/ns/rss/1.0'

#Set this if you want to use a custom Theme Directory
THEME_DIRECTORY = ""
if THEME_DIRECTORY == "":
	if path.isdir('/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME
	elif path.isdir('/usr/share/kiba-dock/icons/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/usr/share/kiba-dock/icons/' + PLUGIN_NAME
	else:
		print "Kiba-Weather: No theme found, exiting"
		exit()
THEME=THEME_DIRECTORY

if USE_METRIC == 1:
	TEMP_SYMBOL = 'C'
else:
	TEMP_SYMBOL = 'F'

class KibaWeather(object):
	weather=''
	dark=0
	icon=THEME + "/sun.png"
	desc="weather"
	temp=""
	icon_added=0
	
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(CHECK_INTERVALL, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		self.get_weather()
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.add_kiba_icon()
			self.set_icon_and_titel()
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "NULL", self.icon)
		
		self.icon_added=1
		
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
		self.kiba.LaunchCommand("%s %s%s" % (BROWSER, URL, ZIP))
	
	def close_handler(self, sender=None):
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.add_kiba_icon()
	
	def get_weather(self):
		dom = minidom.parse(urllib.urlopen(WEATHER_URL))
		condition = dom.getElementsByTagNameNS(WEATHER_NS, 'condition')[0]
		astronomy = dom.getElementsByTagNameNS(WEATHER_NS, 'astronomy')[0]
		
		risetime = astronomy.getAttribute('sunrise').split()[0]
		settime = astronomy.getAttribute('sunset').split()[0]
		risehr = risetime.split(':')[0]
		risemin = risetime.split(':')[1]
		sethr = settime.split(':')[0]
		setmin = settime.split(':')[1]
		
		self.desc = condition.getAttribute('text')
		self.temp = int(condition.getAttribute('temp'))
		if USE_METRIC == 1:
			self.temp = (self.temp - 32) * 5/9
		code = int(condition.getAttribute('code'))
		time = condition.getAttribute('date').split()[4]
		ampm = condition.getAttribute('date').split()[5]
		hour = time.split(':')[0]
		min = time.split(':')[1]
		
		#Set weather
		if code < 13 :
			self.weather = "rain"
		elif code < 19 :
			self.weather = "snow"
		elif code < 23 :
			self.weather = "fog"
		elif code < 31 :
			self.weather = "clouds"
		elif code < 35 :
			self.weather = "clear"
		elif code == 35 :
			self.weather = "rain"
		elif code == 36 :
			self.weather = "clear"
		elif code < 41 :
			self.weather = "rain"
		elif code < 44 :
			self.weather = "snow"
		elif code == 44 :
			self.weather = "clouds"
		elif code == 45 :
			self.weather = "rain"
		elif code == 46 :
			self.weather = "snow"
		elif code == 47 :
			self.weather = "rain"
		
		#Set daytime
		if ampm == "am" :
			if int(hour) == 12 and int(risehr) < 12 :
				self.dark = 1
			elif int(risehr) > int(hour) :
				self.dark = 1
			elif int(risehr) < int(hour) :
				self.dark = 0
			else :
				if int(risemin) > int(min) :
					self.dark = 1
				elif int(risemin) < int(min) :
					self.dark = 0
				else :
					self.dark = -1
		
		elif ampm == "pm" :
			if int(hour) == 12 and int(sethr) < 12 :
				self.dark = 0
			elif int(sethr) > int(hour) :
				self.dark = 0
			elif int(sethr) < int(hour) :
				self.dark = 1
			else :
				if int(setmin) > int(min) :
					self.dark = 0
				elif int(setmin) < int(min) :
					self.dark = 1
				else :
					self.dark = -1
	
	def set_icon_and_titel(self):
		if self.dark == 1 and self.weather == "rain" :
			icon = "/nightrain.png"
		elif self.dark == 1 and self.weather == "snow" :
			icon = "/nightsnow.png"
		elif self.dark == 1 and self.weather == "fog" :
			icon = "/nightfog.png"
		elif self.dark == 1 and self.weather == "clouds" :
			icon = "/nightcloud.png"
		elif self.dark == 1 and self.weather == "clear" :
			icon = "/moon.png"
		elif self.dark < 1 and self.weather == "rain" :
			icon = "/dayrain.png"
		elif self.dark < 1 and self.weather == "snow" :
			icon = "/daysnow.png"
		elif self.dark < 1 and self.weather == "fog" :
			icon = "/dayfog.png"
		elif self.dark < 1 and self.weather == "clouds" :
			icon = "/daycloud.png"
		elif self.dark < 1 and self.weather == "clear" :
			icon = "/sun.png"
		
		self.icon = THEME + icon
		self.kiba.SetIcon(PLUGIN_NAME, self.icon)
		self.kiba.SetTitel(PLUGIN_NAME, "%s - %i%s" % (self.desc, self.temp, TEMP_SYMBOL))
	
	def timeout_func(self):
		if self.icon_added == 0:
			return 1
		
		self.get_weather()
		self.set_icon_and_titel()
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_weather=KibaWeather()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_weather.kiba.RemoveObject(PLUGIN_NAME)
